@@ -15,7 +15,7 @@
 
 #include "Ecore_Config.h"
 #include "Ecore.h"
-
+#include "ecore_config_private.h"
 typedef struct __Ecore_Config_Arg_Callback _Ecore_Config_Arg_Callback;
 struct __Ecore_Config_Arg_Callback
 {
@@ -531,9 +531,9 @@ ecore_config_args_display(void)
    _Ecore_Config_Arg_Callback *callbacks;
 
    if (__ecore_config_app_description)
-      printf("%s\n\n", __ecore_config_app_description);
-   printf("Supported Options:\n");
-   printf(" -h, --help\t       Print this text\n");
+      ERR("%s\n\n", __ecore_config_app_description);
+   ERR("Supported Options:");
+   ERR(" -h, --help\t       Print this text");
    if (!__ecore_config_bundle_local)
       return;
    props = __ecore_config_bundle_local->data;
@@ -545,7 +545,7 @@ ecore_config_args_display(void)
 	     props = props->next;
 	     continue;
 	  }
-	printf(" %c%c%c --%s\t%s %s\n", props->short_opt ? '-' : ' ',
+	INF(" %c%c%c --%s\t%s %s", props->short_opt ? '-' : ' ',
 	       props->short_opt ? props->short_opt : ' ',
 	       props->short_opt ? ',' : ' ',
 	       props->long_opt ? props->long_opt : props->key,
@@ -558,7 +558,7 @@ ecore_config_args_display(void)
    callbacks = _ecore_config_arg_callbacks;
    while (callbacks)
      {
-        printf(" %c%c%c --%s\t%s %s\n", callbacks->short_opt ? '-' : ' ',
+        INF(" %c%c%c --%s\t%s %s", callbacks->short_opt ? '-' : ' ',
 	       callbacks->short_opt ? callbacks->short_opt : ' ',
 	       callbacks->short_opt ? ',' : ' ',
 	       callbacks->long_opt ? callbacks->long_opt : "",
@@ -577,9 +577,9 @@ ecore_config_parse_set(Ecore_Config_Prop * prop, char *arg, char *opt,
    if (!arg)
      {
 	if (opt)
-	   printf("Missing expected argument for option --%s\n", opt);
+	   ERR("Missing expected argument for option --%s", opt);
 	else
-	   printf("Missing expected argument for option -%c\n", opt2);
+	   ERR("Missing expected argument for option -%c", opt2);
 	return ECORE_CONFIG_PARSE_EXIT;
      }
    else
@@ -652,7 +652,7 @@ ecore_config_args_parse(void)
 
 	if (*arg != '-')
 	  {
-	     printf("Unexpected attribute \"%s\"\n", arg);
+	     ERR("Unexpected attribute \"%s\"", arg);
 	     nextarg++;
 	     continue;
 	  }
@@ -705,7 +705,7 @@ ecore_config_args_parse(void)
 			      {
 				 if (!argv[++nextarg])
 				   {
-				      printf("Missing expected argument for option --%s\n", long_opt);
+				      ERR("Missing expected argument for option --%s", long_opt);
 				      return ECORE_CONFIG_PARSE_EXIT;
 				   }
 				   callback->func(argv[nextarg], callback->data);
@@ -717,8 +717,8 @@ ecore_config_args_parse(void)
 	       }
 	     if (!found)
 	       {
-		  printf("Unrecognised option \"%s\"\n", long_opt);
-		  printf("Try using -h or --help for more information.\n\n");
+		  ERR("Unrecognised option \"%s\"", long_opt);
+		  ERR("Try using -h or --help for more information.\n");
 		  return ECORE_CONFIG_PARSE_EXIT;
 	       }
 	  }
@@ -768,7 +768,7 @@ ecore_config_args_parse(void)
 					{
 					   if (!argv[++nextarg])
 					     {
-						printf("Missing expected argument for option -%c\n", short_opt);
+						ERR("Missing expected argument for option -%c", short_opt);
 						return ECORE_CONFIG_PARSE_EXIT;
 					     }
 					   callback->func(argv[nextarg], callback->data);
@@ -780,9 +780,8 @@ ecore_config_args_parse(void)
 			 }
 		       if (!found)
 			 {
-			    printf("Unrecognised option '%c'\n", short_opt);
-			    printf
-			       ("Try using -h or --help for more information.\n\n");
+			    ERR("Unrecognised option '%c'", short_opt);
+			    ERR("Try using -h or --help for more information.\n");
 			    return ECORE_CONFIG_PARSE_EXIT;
 			 }
 		    }
