@@ -93,12 +93,9 @@ _ecore_config_ipc_prop_desc(Ecore_Config_Server * srv, const long serial,
 			    const char *key)
 {
 #ifdef HAVE_EVAS2
-   Ecore_Config_Bundle *theme;
    Ecore_Config_Prop  *e;
 
-   theme = ecore_config_bundle_by_serial_get(srv, serial);
    e = ecore_config_get(key);
-
    if (e)
      {
 	estring            *s = estring_new(512);
@@ -118,11 +115,8 @@ _ecore_config_ipc_prop_get(Ecore_Config_Server * srv, const long serial,
 {
 #ifdef HAVE_EVAS2
    char               *ret;
-   Ecore_Config_Bundle *theme;
 
-   ret = NULL;
-   theme = ecore_config_bundle_by_serial_get(srv, serial);
-   if ((ret = ecore_config_as_string_get( /*theme, */ key)))
+   if ((ret = ecore_config_as_string_get(key)))
       return ret;
 #endif
    return strdup("<undefined>");
@@ -138,8 +132,8 @@ _ecore_config_ipc_prop_set(Ecore_Config_Server * srv, const long serial,
 
    theme = ecore_config_bundle_by_serial_get(srv, serial);
    ret = ecore_config_set(key, (char *)val);
-   E(1, "ipc.prop.set(%s->%s,\"%s\") => %d\n", theme ? theme->identifier : "",
-     key, val, ret);
+   ERR("ipc.prop.set(%s->%s,\"%s\") => %d\n", theme ? theme->identifier : "",
+       key, val, ret);
    return ret;
 #else
    return ECORE_CONFIG_ERR_NOTSUPP;
@@ -265,11 +259,11 @@ _ecore_config_ipc_init(const char *pipe_name)
    memset(list, 0, sizeof(Ecore_Config_Server));
    if ((ret = _ecore_config_ipc_ecore_init(pipe_name, &list->server)) != ECORE_CONFIG_ERR_SUCC)
      {
-	E(2, "_ecore_config_ipc_init: failed to register %s, code %d\n",
+	ERR("_ecore_config_ipc_init: failed to register %s, code %d",
 	  pipe_name, ret);
      }
 
-   E(2, "_ecore_config_ipc_init: registered \"%s\"...\n", pipe_name);
+   ERR("_ecore_config_ipc_init: registered \"%s\"...", pipe_name);
 
    list->name = strdup(pipe_name);
    list->next = __ecore_config_servers;
