@@ -9,6 +9,7 @@
 #include <stdlib.h> /* for NULL */
 
 #include <Ecore.h>
+#include "ecore_private.h"
 #ifdef BUILD_ECORE_EVAS_WIN32
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
@@ -17,7 +18,6 @@
 # include <ecore_win32_private.h>
 #endif /* BUILD_ECORE_EVAS_WIN32 */
 
-#include "ecore_private.h"
 #include "ecore_evas_private.h"
 #include "Ecore_Evas.h"
 
@@ -26,34 +26,32 @@
 #define ECORE_EVAS_EVENT_COUNT 8
 
 static int _ecore_evas_init_count = 0;
-static int _ecore_evas_fps_debug  = 0;
 
 static Ecore_Event_Handler *ecore_evas_event_handlers[ECORE_EVAS_EVENT_COUNT];
-static Ecore_Idle_Enterer  *ecore_evas_idle_enterer = NULL;
-static Ecore_Evas          *ecore_evases = NULL;
 
-static int _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_win32_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_win32_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event);
 
 /* Private functions */
 
-static void
+static int
 _ecore_evas_win32_render(Ecore_Evas *ee)
 {
-   Eina_List *updates;
+   int rend = 0;
+   Eina_List *updates = NULL;
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
    Eina_List *ll;
    Ecore_Evas *ee2;
@@ -61,7 +59,7 @@ _ecore_evas_win32_render(Ecore_Evas *ee)
    EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
      {
         if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-        _ecore_evas_buffer_render(ee2);
+        rend |= _ecore_evas_buffer_render(ee2);
         if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
 #endif
@@ -88,31 +86,9 @@ _ecore_evas_win32_render(Ecore_Evas *ee)
      }
    else
      evas_norender(ee->evas);
+   if (updates) rend = 1;
    if (ee->func.fn_post_render) ee->func.fn_post_render(ee);
-}
-
-static int
-_ecore_evas_win32_idle_enter(void *data __UNUSED__)
-{
-   Ecore_Evas *ee;
-   double       t1 = 0.0;
-   double       t2 = 0.0;
-
-   if (!ecore_evases) return 1;
-   if (_ecore_evas_fps_debug)
-     {
-        t1 = ecore_time_get();
-     }
-   EINA_INLIST_FOREACH(ecore_evases, ee)
-        _ecore_evas_win32_render(ee);
-
-   /*    ecore_x_flush(); */
-   if (_ecore_evas_fps_debug)
-     {
-        t2 = ecore_time_get();
-/*        _ecore_evas_fps_debug_rendertime_add(t2 - t1); */
-     }
-   return 1;
+   return rend;
 }
 
 static int
@@ -121,12 +97,7 @@ _ecore_evas_win32_init(void)
    _ecore_evas_init_count++;
    if (_ecore_evas_init_count > 1)
      return _ecore_evas_init_count;
-
-   if (getenv("ECORE_EVAS_FPS_DEBUG"))
-     _ecore_evas_fps_debug = 1;
-
-   ecore_evas_idle_enterer = ecore_idle_enterer_add(_ecore_evas_win32_idle_enter, NULL);
-
+   
    ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_WIN32_EVENT_MOUSE_IN, _ecore_evas_win32_event_mouse_in, NULL);
    ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_WIN32_EVENT_MOUSE_OUT, _ecore_evas_win32_event_mouse_out, NULL);
    ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WIN32_EVENT_WINDOW_DAMAGE, _ecore_evas_win32_event_window_damage, NULL);
@@ -148,11 +119,8 @@ _ecore_evas_win32_shutdown(void)
      {
         int i;
 
-        while (ecore_evases) _ecore_evas_free(ecore_evases);
         for (i = 0; i < ECORE_EVAS_EVENT_COUNT; i++)
           ecore_event_handler_del(ecore_evas_event_handlers[i]);
-        ecore_idle_enterer_del(ecore_evas_idle_enterer);
-        ecore_evas_idle_enterer = NULL;
         ecore_event_evas_shutdown();
      }
 
@@ -161,7 +129,7 @@ _ecore_evas_win32_shutdown(void)
    return _ecore_evas_init_count;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                 *ee;
@@ -178,12 +146,13 @@ _ecore_evas_win32_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, voi
    /* FIXME to do */
 /*    _ecore_evas_x_modifier_locks_update(ee, e->modifiers); */
    evas_event_feed_mouse_in(ee->evas, e->time, NULL);
+   evas_focus_in(ee->evas);
    _ecore_evas_mouse_move_process(ee, e->x, e->y, e->time);
 
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                  *ee;
@@ -207,7 +176,7 @@ _ecore_evas_win32_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, vo
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                      *ee;
@@ -259,7 +228,7 @@ _ecore_evas_win32_event_window_damage(void *data __UNUSED__, int type __UNUSED__
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                       *ee;
@@ -277,7 +246,7 @@ _ecore_evas_win32_event_window_destroy(void *data __UNUSED__, int type __UNUSED_
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                    *ee;
@@ -296,7 +265,7 @@ _ecore_evas_win32_event_window_show(void *data __UNUSED__, int type __UNUSED__,
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                    *ee;
@@ -315,7 +284,7 @@ _ecore_evas_win32_event_window_hide(void *data __UNUSED__, int type __UNUSED__,
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                         *ee;
@@ -373,7 +342,7 @@ _ecore_evas_win32_event_window_configure(void *data __UNUSED__, int type __UNUSE
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_win32_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                              *ee;
@@ -401,7 +370,6 @@ _ecore_evas_win32_free(Ecore_Evas *ee)
 
    ecore_win32_window_free((struct _Ecore_Win32_Window *)ee->prop.window);
    ecore_event_window_unregister(ee->prop.window);
-   ecore_evases = (Ecore_Evas *) eina_inlist_remove(EINA_INLIST_GET(ecore_evases), EINA_INLIST_GET(ee));
    _ecore_evas_win32_shutdown();
    ecore_win32_shutdown();
 }
@@ -562,7 +530,7 @@ _ecore_evas_win32_rotation_set_internal(Ecore_Evas *ee, int rotation)
 }
 
 static void
-_ecore_evas_win32_rotation_set(Ecore_Evas *ee, int rotation)
+_ecore_evas_win32_rotation_set(Ecore_Evas *ee, int rotation, int resize)
 {
    INF("ecore evas rotation: %s", rotation ? "yes" : "no");
 
@@ -835,7 +803,7 @@ _ecore_evas_win32_fullscreen_set(Ecore_Evas *ee, int on)
 }
 
 
-static const Ecore_Evas_Engine_Func _ecore_win32_engine_func =
+static Ecore_Evas_Engine_Func _ecore_win32_engine_func =
 {
    _ecore_evas_win32_free,
      NULL,
@@ -881,7 +849,10 @@ static const Ecore_Evas_Engine_Func _ecore_win32_engine_func =
      NULL, /* _ecore_evas_x_withdrawn_set */
      NULL, /* _ecore_evas_x_sticky_set */
      NULL, /* _ecore_evas_x_ignore_events_set */
-     NULL  /* _ecore_evas_x_alpha_set */
+     NULL,  /* _ecore_evas_x_alpha_set */
+     NULL, //transparent
+     
+     NULL // render
 };
 
 #endif /* BUILD_ECORE_EVAS_WIN32 */
@@ -1108,9 +1079,10 @@ _ecore_evas_win32_new_internal(int (*_ecore_evas_engine_init)(Ecore_Evas *ee),
         return NULL;
      }
 
-   ecore_evases = (Ecore_Evas *) eina_inlist_prepend(EINA_INLIST_GET(ecore_evases), EINA_INLIST_GET(ee));
+   ee->engine.func->fn_render = _ecore_evas_win32_render;
+   _ecore_evas_register(ee);
    ecore_event_window_register(ee->prop.window, ee, ee->evas, _ecore_evas_mouse_move_process);
-
+   
    return ee;
 }
 
