@@ -9,6 +9,7 @@
 #include <stdlib.h> /* for NULL */
 
 #include <Ecore.h>
+#include "ecore_private.h"
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_16_WINCE
 # define WIN32_LEAN_AND_MEAN
 # include <windows.h>
@@ -17,7 +18,6 @@
 # include <ecore_wince_private.h>
 #endif /* BUILD_ECORE_EVAS_SOFTWARE_16_WINCE */
 
-#include "ecore_private.h"
 #include "ecore_evas_private.h"
 #include "Ecore_Evas.h"
 
@@ -26,31 +26,29 @@
 #define ECORE_EVAS_EVENT_COUNT 7
 
 static int _ecore_evas_init_count = 0;
-static int _ecore_evas_fps_debug  = 0;
 
 static Ecore_Event_Handler *ecore_evas_event_handlers[ECORE_EVAS_EVENT_COUNT];
-static Ecore_Idle_Enterer  *ecore_evas_idle_enterer = NULL;
-static Ecore_Evas          *ecore_evases = NULL;
 
-static int _ecore_evas_wince_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event);
 
-static int _ecore_evas_wince_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event);
+static Eina_Bool _ecore_evas_wince_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event);
 
 /* Private functions */
 
-static void
+static int
 _ecore_evas_wince_render(Ecore_Evas *ee)
 {
+   int rend = 0;
    Eina_List *updates = NULL;
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
    Eina_List *ll;
@@ -59,7 +57,7 @@ _ecore_evas_wince_render(Ecore_Evas *ee)
    EINA_LIST_FOREACH(ee->sub_ecore_evas, ll, ee2)
      {
 	if (ee2->func.fn_pre_render) ee2->func.fn_pre_render(ee2);
-	_ecore_evas_buffer_render(ee2);
+	rend |= _ecore_evas_buffer_render(ee2);
 	if (ee2->func.fn_post_render) ee2->func.fn_post_render(ee2);
      }
 #endif
@@ -86,30 +84,9 @@ _ecore_evas_wince_render(Ecore_Evas *ee)
      }
    else
      evas_norender(ee->evas);
+   if (updates) rend = 1;
    if (ee->func.fn_post_render) ee->func.fn_post_render(ee);
-}
-
-static int
-_ecore_evas_wince_idle_enter(void *data __UNUSED__)
-{
-   Ecore_Evas *ee;
-   double       t1 = 0.0;
-   double       t2 = 0.0;
-
-   if (!ecore_evases) return 1;
-   if (_ecore_evas_fps_debug)
-     {
-	t1 = ecore_time_get();
-     }
-   EINA_INLIST_FOREACH(ecore_evases, ee)
-	_ecore_evas_wince_render(ee);
-
-   if (_ecore_evas_fps_debug)
-     {
-	t2 = ecore_time_get();
-	_ecore_evas_fps_debug_rendertime_add(t2 - t1);
-     }
-   return 1;
+   return rend;
 }
 
 static int
@@ -118,12 +95,7 @@ _ecore_evas_wince_init(void)
    _ecore_evas_init_count++;
    if (_ecore_evas_init_count > 1)
      return _ecore_evas_init_count;
-
-   if (getenv("ECORE_EVAS_FPS_DEBUG"))
-     _ecore_evas_fps_debug = 1;
-
-   ecore_evas_idle_enterer = ecore_idle_enterer_add(_ecore_evas_wince_idle_enter, NULL);
-
+   
    ecore_evas_event_handlers[0]  = ecore_event_handler_add(ECORE_WINCE_EVENT_MOUSE_IN, _ecore_evas_wince_event_mouse_in, NULL);
    ecore_evas_event_handlers[1]  = ecore_event_handler_add(ECORE_WINCE_EVENT_MOUSE_OUT, _ecore_evas_wince_event_mouse_out, NULL);
    ecore_evas_event_handlers[2]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DAMAGE, _ecore_evas_wince_event_window_damage, NULL);
@@ -132,8 +104,6 @@ _ecore_evas_wince_init(void)
    ecore_evas_event_handlers[5]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_HIDE, _ecore_evas_wince_event_window_hide, NULL);
    ecore_evas_event_handlers[6]  = ecore_event_handler_add(ECORE_WINCE_EVENT_WINDOW_DELETE_REQUEST, _ecore_evas_wince_event_window_delete_request, NULL);
 
-   if (_ecore_evas_fps_debug) _ecore_evas_fps_debug_init();
-
    ecore_event_evas_init();
    return _ecore_evas_init_count;
 }
@@ -146,12 +116,8 @@ _ecore_evas_wince_shutdown(void)
      {
 	int i;
 
-	while (ecore_evases) _ecore_evas_free(ecore_evases);
 	for (i = 0; i < ECORE_EVAS_EVENT_COUNT; i++)
 	  ecore_event_handler_del(ecore_evas_event_handlers[i]);
-	ecore_idle_enterer_del(ecore_evas_idle_enterer);
-	ecore_evas_idle_enterer = NULL;
-	if (_ecore_evas_fps_debug) _ecore_evas_fps_debug_shutdown();
         ecore_event_evas_shutdown();
      }
 
@@ -160,7 +126,7 @@ _ecore_evas_wince_shutdown(void)
    return _ecore_evas_init_count;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                 *ee;
@@ -182,7 +148,7 @@ _ecore_evas_wince_event_mouse_in(void *data __UNUSED__, int type __UNUSED__, voi
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                  *ee;
@@ -206,7 +172,7 @@ _ecore_evas_wince_event_mouse_out(void *data __UNUSED__, int type __UNUSED__, vo
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                      *ee;
@@ -254,7 +220,7 @@ _ecore_evas_wince_event_window_damage(void *data __UNUSED__, int type __UNUSED__
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_window_destroy(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                       *ee;
@@ -272,7 +238,7 @@ _ecore_evas_wince_event_window_destroy(void *data __UNUSED__, int type __UNUSED_
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_window_show(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                    *ee;
@@ -291,7 +257,7 @@ _ecore_evas_wince_event_window_show(void *data __UNUSED__, int type __UNUSED__,
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_window_hide(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                    *ee;
@@ -310,7 +276,7 @@ _ecore_evas_wince_event_window_hide(void *data __UNUSED__, int type __UNUSED__,
    return 1;
 }
 
-static int
+static Eina_Bool
 _ecore_evas_wince_event_window_delete_request(void *data __UNUSED__, int type __UNUSED__, void *event)
 {
    Ecore_Evas                              *ee;
@@ -337,7 +303,6 @@ _ecore_evas_wince_free(Ecore_Evas *ee)
 
    ecore_wince_window_free((Ecore_WinCE_Window *)ee->prop.window);
    ecore_event_window_unregister(ee->prop.window);
-   ecore_evases = (Ecore_Evas *) eina_inlist_remove(EINA_INLIST_GET(ecore_evases), EINA_INLIST_GET(ee));
    _ecore_evas_wince_shutdown();
    ecore_wince_shutdown();
 }
@@ -708,7 +673,7 @@ _ecore_evas_wince_fullscreen_set(Ecore_Evas *ee, int on)
      }
 }
 
-static const Ecore_Evas_Engine_Func _ecore_wince_engine_func =
+static Ecore_Evas_Engine_Func _ecore_wince_engine_func =
 {
    _ecore_evas_wince_free,
    NULL,
@@ -754,7 +719,10 @@ static const Ecore_Evas_Engine_Func _ecore_wince_engine_func =
    NULL, /* _ecore_evas_x_withdrawn_set */
    NULL, /* _ecore_evas_x_sticky_set */
    NULL, /* _ecore_evas_x_ignore_events_set */
-   NULL  /* _ecore_evas_x_alpha_set */
+   NULL, /* _ecore_evas_x_alpha_set */
+   NULL, //transparent
+     
+     NULL // render
 };
 
 /* API */
@@ -848,8 +816,10 @@ ecore_evas_software_wince_new_internal(int                 backend,
         ecore_wince_window_resume_set((Ecore_WinCE_Window *)ee->prop.window, einfo->func.resume);
      }
 
-   ecore_evases = (Ecore_Evas *) eina_inlist_prepend(EINA_INLIST_GET(ecore_evases), EINA_INLIST_GET(ee));
+   ee->engine.func->fn_render = _ecore_evas_wince_render;
+   _ecore_evas_register(ee);
    ecore_event_window_register(ee->prop.window, ee, ee->evas, (Ecore_Event_Mouse_Move_Cb)_ecore_evas_mouse_move_process);
+   evas_focus_in(ee->evas);
 
    return ee;
 }
