@@ -1,11 +1,11 @@
 #ifndef _ECORE_FB_PRIVATE_H
 #define _ECORE_FB_PRIVATE_H
 
-#include "ecore_private.h"
 #include "Ecore.h"
-#include "Ecore_Data.h"
+#include "ecore_private.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <unistd.h>
 #include <termios.h>
 #include <sys/types.h>
@@ -19,8 +19,8 @@
  #define kernel_ulong_t unsigned long 
  #define BITS_PER_LONG 32
  #include <linux/input.h>
- #undef kernel_ulong_t <-added
- #undef BITS_PER_LONG <-added
+ #undef kernel_ulong_t
+ #undef BITS_PER_LONG
 #else
  #include <linux/input.h>
 #endif
@@ -52,7 +52,7 @@ struct _Ecore_Fb_Input_Device
 		/* absolute axis */
 		int min_w, min_h;
 		double rel_w, rel_h;
-
+		int event;
 	} mouse;
 	struct
 	{
