@@ -6,14 +6,17 @@
 # include <config.h>
 #endif
 
-#include <Ecore_Str.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
 #include <dirent.h>
 
+#ifndef _MSC_VER
+# include <unistd.h>
+# include <libgen.h>
+#endif
+
 #ifndef _FILE_OFFSET_BITS
 # define _FILE_OFFSET_BITS  64
 #endif
@@ -26,7 +29,7 @@
 
 #include "ecore_file_private.h"
 
-
+int _ecore_file_log_dom = -1;
 static int _ecore_file_init_count = 0;
 
 /* externally accessible functions */
@@ -41,7 +44,12 @@ ecore_file_init()
 {
    if (++_ecore_file_init_count != 1)
      return _ecore_file_init_count;
-
+   _ecore_file_log_dom = eina_log_domain_register("EcoreFile", ECORE_FILE_DEFAULT_LOG_COLOR);
+   if(_ecore_file_log_dom < 0) 
+     {
+       EINA_LOG_ERR("Impossible to create a log domain for the ecore file module.");
+       return --_ecore_file_init_count;
+     }
    ecore_file_path_init();
    ecore_file_monitor_init();
    ecore_file_download_init();
@@ -81,7 +89,8 @@ ecore_file_shutdown()
    ecore_file_download_shutdown();
    ecore_file_monitor_shutdown();
    ecore_file_path_shutdown();
-
+   eina_log_domain_unregister(_ecore_file_log_dom);
+   _ecore_file_log_dom = -1;
    return _ecore_file_init_count;
 }
 
@@ -117,31 +126,31 @@ ecore_file_size(const char *file)
 /**
  * Check if file exists
  * @param  file The name of the file
- * @return 1 if file exists on local filesystem, 0 otherwise
+ * @return EINA_TRUE if file exists on local filesystem, EINA_FALSE otherwise
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_exists(const char *file)
 {
    struct stat st;
 
    /*Workaround so that "/" returns a true, otherwise we can't monitor "/" in ecore_file_monitor*/
-   if (stat(file, &st) < 0 && strcmp(file, "/")) return 0;
-   return 1;
+   if (stat(file, &st) < 0 && strcmp(file, "/")) return EINA_FALSE;
+   return EINA_TRUE;
 }
 
 /**
  * Check if file is a directory
  * @param  file The name of the file
- * @return 1 if file exist and is a directory, 0 otherwise
+ * @return EINA_TRUE if file exist and is a directory, EINA_FALSE otherwise
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_is_dir(const char *file)
 {
    struct stat st;
 
-   if (stat(file, &st) < 0) return 0;
-   if (S_ISDIR(st.st_mode)) return 1;
-   return 0;
+   if (stat(file, &st) < 0) return EINA_FALSE;
+   if (S_ISDIR(st.st_mode)) return EINA_TRUE;
+   return EINA_FALSE;
 }
 
 static mode_t default_mode = S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
@@ -149,15 +158,15 @@ static mode_t default_mode = S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S
 /**
  * Create a new directory
  * @param  dir The name of the directory to create
- * @return 1 on successfull creation, 0 on failure
+ * @return EINA_TRUE on successfull creation, EINA_FALSE on failure
  *
  * The directory is created with the mode: S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_mkdir(const char *dir)
 {
-   if (mkdir(dir, default_mode) < 0) return 0;
-   return 1;
+   if (mkdir(dir, default_mode) < 0) return EINA_FALSE;
+   return EINA_TRUE;
 }
 
 /**
@@ -171,8 +180,10 @@ ecore_file_mkdir(const char *dir)
 EAPI int
 ecore_file_mkdirs(const char **dirs)
 {
-   if (!dirs) return -1;
    int i = 0;
+
+   if (!dirs) return -1;
+
    for (; *dirs != NULL; dirs++)
      if (ecore_file_mkdir(*dirs))
        i++;
@@ -213,7 +224,7 @@ ecore_file_mksubdirs(const char *base, const char **subdirs)
      return 0;
 
 #ifndef HAVE_ATFILE_SOURCE
-   baselen = ecore_strlcpy(buf, base, sizeof(buf));
+   baselen = eina_strlcpy(buf, base, sizeof(buf));
    if ((baselen < 1) || (baselen + 1 >= (int)sizeof(buf)))
      return 0;
 
@@ -235,7 +246,7 @@ ecore_file_mksubdirs(const char *base, const char **subdirs)
 	struct stat st;
 
 #ifndef HAVE_ATFILE_SOURCE
-	ecore_strlcpy(buf + baselen, *subdirs, sizeof(buf) - baselen);
+	eina_strlcpy(buf + baselen, *subdirs, sizeof(buf) - baselen);
 	if (stat(buf, &st) == 0)
 #else
 	if (fstatat(fd, *subdirs, &st, 0) == 0)
@@ -274,54 +285,63 @@ ecore_file_mksubdirs(const char *base, const char **subdirs)
 /**
  * Delete the given dir
  * @param  dir The name of the directory to delete
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_rmdir(const char *dir)
 {
-   if (rmdir(dir) < 0) return 0;
-   return 1;
+   if (rmdir(dir) < 0) return EINA_FALSE;
+   return EINA_TRUE;
 }
 
 /**
  * Delete the given file
  * @param  file The name of the file to delete
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_unlink(const char *file)
 {
-   if (unlink(file) < 0) return 0;
-   return 1;
+   if (unlink(file) < 0) return EINA_FALSE;
+   return EINA_TRUE;
+}
+
+/**
+ * Remove the given file or directory
+ * @param  file The name of the file or directory to delete
+ * @return EINA_TRUE on success, EINA_FALSE on failure
+ */
+EAPI Eina_Bool
+ecore_file_remove(const char *file)
+{
+   if (remove(file) < 0) return EINA_FALSE;
+   return EINA_TRUE;
 }
 
 /**
  * Delete a directory and all its contents
  * @param  dir The name of the directory to delete
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  *
  * If dir is a link only the link is removed
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_recursive_rm(const char *dir)
 {
-   DIR                *dirp;
-   struct dirent      *dp;
-   char               path[PATH_MAX];
-   char               buf[PATH_MAX];
-   struct             stat st;
-   int                ret;
+   DIR *dirp;
+   struct dirent *dp;
+   char path[PATH_MAX], buf[PATH_MAX];
+   struct stat st;
+   int ret;
 
    if (readlink(dir, buf, sizeof(buf)) > 0)
-     {
-	return ecore_file_unlink(dir);
-     }
+     return ecore_file_unlink(dir);
 
    ret = stat(dir, &st);
    if ((ret == 0) && (S_ISDIR(st.st_mode)))
      {
 	ret = 1;
-	if (stat(dir, &st) == -1) return 0;
+	if (stat(dir, &st) == -1) return EINA_FALSE;
 	dirp = opendir(dir);
 	if (dirp)
 	  {
@@ -337,53 +357,55 @@ ecore_file_recursive_rm(const char *dir)
 	     closedir(dirp);
 	  }
 	if (!ecore_file_rmdir(dir)) ret = 0;
-        return ret;
+        if (ret)
+            return EINA_TRUE;
+        else
+            return EINA_FALSE;
      }
    else
      {
-	if (ret == -1) return 0;
+	if (ret == -1) return EINA_FALSE;
 	return ecore_file_unlink(dir);
      }
-
-   return 1;
 }
 
-static inline int
+static inline Eina_Bool
 _ecore_file_mkpath_if_not_exists(const char *path)
 {
    struct stat st;
+
    if (stat(path, &st) < 0)
      return ecore_file_mkdir(path);
    else if (!S_ISDIR(st.st_mode))
-     return 0;
+     return EINA_FALSE;
    else
-     return 1;
+     return EINA_TRUE;
 }
 
 /**
  * Create a complete path
  * @param  path The path to create
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  *
  * @see ecore_file_mkpaths() and ecore_file_mkdir()
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_mkpath(const char *path)
 {
    char ss[PATH_MAX];
    unsigned int i;
 
    if (ecore_file_is_dir(path))
-     return 1;
+     return EINA_TRUE;
 
    for (i = 0; path[i] != '\0'; ss[i] = path[i], i++)
      {
-	if (i == sizeof(ss) - 1) return 0;
+	if (i == sizeof(ss) - 1) return EINA_FALSE;
 	if ((path[i] == '/') && (i > 0))
 	  {
 	     ss[i] = '\0';
 	     if (!_ecore_file_mkpath_if_not_exists(ss))
-	       return 0;
+	       return EINA_FALSE;
 	  }
      }
    ss[i] = '\0';
@@ -401,8 +423,10 @@ ecore_file_mkpath(const char *path)
 EAPI int
 ecore_file_mkpaths(const char **paths)
 {
-   if (!paths) return -1;
    int i = 0;
+
+   if (!paths) return -1;
+
    for (; *paths != NULL; paths++)
      if (ecore_file_mkpath(*paths))
        i++;
@@ -413,32 +437,31 @@ ecore_file_mkpaths(const char **paths)
  * Copy a file
  * @param  src The name of the source file
  * @param  dst The name of the destination file
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_cp(const char *src, const char *dst)
 {
-   FILE               *f1, *f2;
-   char                buf[16384];
-   char                realpath1[PATH_MAX];
-   char                realpath2[PATH_MAX];
-   size_t              num;
-   int                 ret = 1;
+   FILE *f1, *f2;
+   char buf[16384];
+   char realpath1[PATH_MAX], realpath2[PATH_MAX];
+   size_t num;
+   Eina_Bool ret = EINA_TRUE;
 
-   if (!realpath(src, realpath1)) return 0;
-   if (realpath(dst, realpath2) && !strcmp(realpath1, realpath2)) return 0;
+   if (!realpath(src, realpath1)) return EINA_FALSE;
+   if (realpath(dst, realpath2) && !strcmp(realpath1, realpath2)) return EINA_FALSE;
 
    f1 = fopen(src, "rb");
-   if (!f1) return 0;
+   if (!f1) return EINA_FALSE;
    f2 = fopen(dst, "wb");
    if (!f2)
      {
 	fclose(f1);
-	return 0;
+	return EINA_FALSE;
      }
    while ((num = fread(buf, 1, sizeof(buf), f1)) > 0)
      {
-	if (fwrite(buf, 1, num, f2) != num) ret = 0;
+	if (fwrite(buf, 1, num, f2) != num) ret = EINA_FALSE;
      }
    fclose(f1);
    fclose(f2);
@@ -449,9 +472,9 @@ ecore_file_cp(const char *src, const char *dst)
  * Move a file
  * @param  src The name of the source file
  * @param  dst The name of the destination file
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_mv(const char *src, const char *dst)
 {
    char buf[PATH_MAX];
@@ -471,6 +494,7 @@ ecore_file_mv(const char *src, const char *dst)
 	     if (S_ISREG(st.st_mode))
 	       {
 		  char *dir;
+
 		  dir = ecore_file_dir_get(dst);
 		  // Since we can't directly rename, try to 
 		  // copy to temp file in the dst directory
@@ -512,24 +536,24 @@ ecore_file_mv(const char *src, const char *dst)
      }
 
 PASS:
-   return 1;
+   return EINA_TRUE;
 
 FAIL:
-   return 0;
+   return EINA_FALSE;
 }
 
 /**
  * Create a symbolic link
  * @param  src The name of the file to link
  * @param  dest The name of link
- * @return 1 on success, 0 on failure
+ * @return EINA_TRUE on success, EINA_FALSE on failure
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_symlink(const char *src, const char *dest)
 {
-   if (!symlink(src, dest)) return 1;
+   if (!symlink(src, dest)) return EINA_TRUE;
 
-   return 0;
+   return EINA_FALSE;
 }
 
 /**
@@ -541,7 +565,7 @@ ecore_file_symlink(const char *src, const char *dest)
 EAPI char *
 ecore_file_realpath(const char *file)
 {
-   char  buf[PATH_MAX];
+   char buf[PATH_MAX];
 
    /*
     * Some implementations of realpath do not conform to the SUS.
@@ -577,59 +601,53 @@ ecore_file_file_get(const char *path)
 EAPI char *
 ecore_file_dir_get(const char *file)
 {
-   char               *p;
-   char                buf[PATH_MAX];
+   char *p;
+   char buf[PATH_MAX];
 
+   if (!file) return NULL;
    strncpy(buf, file, PATH_MAX);
-   p = strrchr(buf, '/');
-   if (!p)
-     {
-	return strdup(file);
-     }
-
-   if (p == buf) return strdup("/");
-
-   *p = 0;
-   return strdup(buf);
+   buf[PATH_MAX - 1] = 0;
+   p = dirname(buf);
+   return strdup(p);
 }
 
 /**
  * Check if file can be read
  * @param  file The name of the file
- * @return 1 if the file is readable, 0 otherwise
+ * @return EINA_TRUE if the file is readable, EINA_FALSE otherwise
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_can_read(const char *file)
 {
-   if (!file) return 0;
-   if (!access(file, R_OK)) return 1;
-   return 0;
+   if (!file) return EINA_FALSE;
+   if (!access(file, R_OK)) return EINA_TRUE;
+   return EINA_FALSE;
 }
 
 /**
  * Check if file can be written
  * @param  file The name of the file
- * @return 1 if the file is writable, 0 otherwise
+ * @return EINA_TRUE if the file is writable, EINA_FALSE otherwise
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_can_write(const char *file)
 {
-   if (!file) return 0;
-   if (!access(file, W_OK)) return 1;
-   return 0;
+   if (!file) return EINA_FALSE;
+   if (!access(file, W_OK)) return EINA_TRUE;
+   return EINA_FALSE;
 }
 
 /**
  * Check if file can be executed
  * @param  file The name of the file
- * @return 1 if the file can be executed, 0 otherwise
+ * @return EINA_TRUE if the file can be executed, EINA_FALSE otherwise
  */
-EAPI int
+EAPI Eina_Bool
 ecore_file_can_exec(const char *file)
 {
-   if (!file) return 0;
-   if (!access(file, X_OK)) return 1;
-   return 0;
+   if (!file) return EINA_FALSE;
+   if (!access(file, X_OK)) return EINA_TRUE;
+   return EINA_FALSE;
 }
 
 /**
@@ -640,8 +658,8 @@ ecore_file_can_exec(const char *file)
 EAPI char *
 ecore_file_readlink(const char *link)
 {
-   char                buf[PATH_MAX];
-   int                 count;
+   char buf[PATH_MAX];
+   int count;
 
    if ((count = readlink(link, buf, sizeof(buf))) < 0) return NULL;
    buf[count] = 0;
@@ -661,10 +679,10 @@ ecore_file_readlink(const char *link)
 EAPI Eina_List *
 ecore_file_ls(const char *dir)
 {
-   char               *f;
-   DIR                *dirp;
-   struct dirent      *dp;
-   Eina_List         *list = NULL;
+   char *f;
+   DIR *dirp;
+   struct dirent *dp;
+   Eina_List *list = NULL;
 
    dirp = opendir(dir);
    if (!dirp) return NULL;
@@ -679,7 +697,7 @@ ecore_file_ls(const char *dir)
      }
    closedir(dirp);
 
-   list = eina_list_sort(list, ECORE_SORT_MIN, ECORE_COMPARE_CB(strcoll));
+   list = eina_list_sort(list, eina_list_count(list), EINA_COMPARE_CB(strcoll));
 
    return list;
 }
@@ -884,9 +902,7 @@ ecore_file_strip_ext(const char *path)
 
    p = strrchr(path, '.');
    if (!p)
-     {
-	file = strdup(path);
-     }
+     file = strdup(path);
    else if (p != path)
      {
 	file = malloc(((p - path) + 1) * sizeof(char));
