@@ -6,13 +6,10 @@
 # include <config.h>
 #endif
 
-#include "Ecore.h"
-#include "Ecore_Con.h"
-#include "ecore_private.h"
-#include "ecore_ipc_private.h"
-#include "Ecore_Ipc.h"
+#include <string.h>
 
 #ifdef HAVE_NETINET_IN_H
+# include <sys/types.h>
 # include <netinet/in.h>
 #endif
 
@@ -20,6 +17,12 @@
 # include <winsock2.h>
 #endif
 
+#include "Ecore.h"
+#include "ecore_private.h"
+#include "Ecore_Con.h"
+#include "ecore_ipc_private.h"
+#include "Ecore_Ipc.h"
+
 #define DLT_ZERO   0
 #define DLT_ONE    1
 #define DLT_SAME   2
@@ -37,6 +40,8 @@
 #define DLT_R1     14
 #define DLT_R2     15
 
+int _ecore_ipc_log_dom = -1;
+
 /* byte swappers - for dealing with big vs little endian machines */
 EAPI unsigned short
 _ecore_ipc_swap_16(unsigned short v)
@@ -220,12 +225,12 @@ _ecore_ipc_ddlt_int(int in, int prev, int mode)
    return 0;
 }
 
-static int _ecore_ipc_event_client_add(void *data, int ev_type, void *ev);
-static int _ecore_ipc_event_client_del(void *data, int ev_type, void *ev);
-static int _ecore_ipc_event_server_add(void *data, int ev_type, void *ev);
-static int _ecore_ipc_event_server_del(void *data, int ev_type, void *ev);
-static int _ecore_ipc_event_client_data(void *data, int ev_type, void *ev);
-static int _ecore_ipc_event_server_data(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_client_add(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_client_del(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_server_add(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_server_del(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_client_data(void *data, int ev_type, void *ev);
+static Eina_Bool _ecore_ipc_event_server_data(void *data, int ev_type, void *ev);
 static void _ecore_ipc_event_client_add_free(void *data, void *ev);
 static void _ecore_ipc_event_client_del_free(void *data, void *ev);
 static void _ecore_ipc_event_client_data_free(void *data, void *ev);
@@ -263,7 +268,12 @@ ecore_ipc_init(void)
 
    if (++_ecore_ipc_init_count != 1)
      return _ecore_ipc_init_count;
-
+   _ecore_ipc_log_dom = eina_log_domain_register("EcoreIpc", ECORE_IPC_DEFAULT_LOG_COLOR);
+   if(_ecore_ipc_log_dom < 0)
+     {
+       EINA_LOG_ERR("Impossible to create a log domain for the Ecore IPC module.");
+       return --_ecore_ipc_init_count;
+     }
    if (!ecore_con_init())
      return --_ecore_ipc_init_count;
 
@@ -284,7 +294,7 @@ ecore_ipc_init(void)
 					  _ecore_ipc_event_server_del, NULL);
    handler[i++] = ecore_event_handler_add(ECORE_CON_EVENT_CLIENT_DATA,
 					  _ecore_ipc_event_client_data, NULL);
-   handler[i++] = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DATA,
+   handler[i] = ecore_event_handler_add(ECORE_CON_EVENT_SERVER_DATA,
 					  _ecore_ipc_event_server_data, NULL);
    return _ecore_ipc_init_count;
 }
@@ -309,7 +319,8 @@ ecore_ipc_shutdown(void)
      ecore_event_handler_del(handler[i]);
 
    ecore_con_shutdown();
-
+   eina_log_domain_unregister(_ecore_ipc_log_dom);
+   _ecore_ipc_log_dom = -1;
    return _ecore_ipc_init_count;
 }
 
@@ -999,20 +1010,20 @@ ecore_ipc_ssl_available_get(void)
 }
 
 
-static int
+static Eina_Bool
 _ecore_ipc_event_client_add(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Add *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Client *cl;
 	Ecore_Ipc_Server *svr;
 
 	cl = calloc(1, sizeof(Ecore_Ipc_Client));
-	if (!cl) return 0;
+	if (!cl) return ECORE_CALLBACK_CANCEL;
 	svr = ecore_con_server_data_get(ecore_con_client_server_get(e->client));
 	ECORE_MAGIC_SET(cl, ECORE_MAGIC_IPC_CLIENT);
 	cl->client = e->client;
@@ -1034,16 +1045,16 @@ _ecore_ipc_event_client_add(void *data __UNUSED__, int ev_type __UNUSED__, void
 	       }
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static int
+static Eina_Bool
 _ecore_ipc_event_client_del(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Del *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Client *cl;
@@ -1068,16 +1079,16 @@ _ecore_ipc_event_client_del(void *data __UNUSED__, int ev_type __UNUSED__, void
 	       }
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static int
+static Eina_Bool
 _ecore_ipc_event_server_add(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Server_Add *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Server *svr;
@@ -1097,16 +1108,16 @@ _ecore_ipc_event_server_add(void *data __UNUSED__, int ev_type __UNUSED__, void
 	       }
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
-static int
+static Eina_Bool
 _ecore_ipc_event_server_del(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Server_Del *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Server *svr;
@@ -1126,7 +1137,7 @@ _ecore_ipc_event_server_del(void *data __UNUSED__, int ev_type __UNUSED__, void
 	       }
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 #define CLSZ(_n) \
@@ -1170,13 +1181,13 @@ _ecore_ipc_event_server_del(void *data __UNUSED__, int ev_type __UNUSED__, void
      } \
    msg._member = _ecore_ipc_ddlt_int(d, cl->prev.i._member, md);
 
-static int
+static Eina_Bool
 _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Client_Data *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(ecore_con_client_server_get(e->client)))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Client *cl;
@@ -1200,7 +1211,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		  free(cl->buf);
 		  cl->buf = 0;
 		  cl->buf_size  = 0;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 	     cl->buf = buf;
 	     memcpy(cl->buf + cl->buf_size, e->data, e->size);
@@ -1230,7 +1241,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 	     if ((cl->buf_size - offset) < s)
 	       {
 		  if (offset > 0) goto scroll;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 
 	     s = 4;
@@ -1265,7 +1276,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		       if (msg.size > 0)
 			 {
 			    buf = malloc(msg.size);
-			    if (!buf) return 0;
+			    if (!buf) return ECORE_CALLBACK_CANCEL;
 			    memcpy(buf, cl->buf + offset + s, msg.size);
 			 }
 		       if (!cl->delete_me)
@@ -1295,7 +1306,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		       free(cl->buf);
 		       cl->buf = NULL;
 		       cl->buf_size = 0;
-		       return 0;
+		       return ECORE_CALLBACK_CANCEL;
 		    }
 		  goto redo;
 	       }
@@ -1310,7 +1321,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		  free(cl->buf);
 		  cl->buf = NULL;
 		  cl->buf_size = 0;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 	     memcpy(buf, cl->buf + offset, cl->buf_size - offset);
 	     free(cl->buf);
@@ -1318,7 +1329,7 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 	     cl->buf_size -= offset;
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 #define SVSZ(_n) \
@@ -1362,13 +1373,13 @@ _ecore_ipc_event_client_data(void *data __UNUSED__, int ev_type __UNUSED__, void
      } \
    msg._member = _ecore_ipc_ddlt_int(d, svr->prev.i._member, md);
 
-static int
+static Eina_Bool
 _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void *ev)
 {
    Ecore_Con_Event_Server_Data *e;
 
    e = ev;
-   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return 1;
+   if (!eina_list_data_find(servers, ecore_con_server_data_get(e->server))) return ECORE_CALLBACK_RENEW;
    /* handling code here */
      {
 	Ecore_Ipc_Server *svr;
@@ -1392,7 +1403,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		  free(svr->buf);
 		  svr->buf = 0;
 		  svr->buf_size  = 0;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 	     svr->buf = buf;
 	     memcpy(svr->buf + svr->buf_size, e->data, e->size);
@@ -1422,7 +1433,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 	     if ((svr->buf_size - offset) < s)
 	       {
 		  if (offset > 0) goto scroll;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 
 	     s = 4;
@@ -1446,7 +1457,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		       if (msg.size > 0)
 			 {
 			    buf = malloc(msg.size);
-			    if (!buf) return 0;
+			    if (!buf) return ECORE_CALLBACK_CANCEL;
 			    memcpy(buf, svr->buf + offset + s, msg.size);
 			 }
 		       if (!svr->delete_me)
@@ -1476,7 +1487,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		       free(svr->buf);
 		       svr->buf = NULL;
 		       svr->buf_size = 0;
-		       return 0;
+		       return ECORE_CALLBACK_CANCEL;
 		    }
 		  goto redo;
 	       }
@@ -1491,7 +1502,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 		  free(svr->buf);
 		  svr->buf = NULL;
 		  svr->buf_size = 0;
-		  return 0;
+		  return ECORE_CALLBACK_CANCEL;
 	       }
 	     memcpy(buf, svr->buf + offset, svr->buf_size - offset);
 	     free(svr->buf);
@@ -1499,7 +1510,7 @@ _ecore_ipc_event_server_data(void *data __UNUSED__, int ev_type __UNUSED__, void
 	     svr->buf_size -= offset;
 	  }
      }
-   return 0;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static void
