@@ -12,8 +12,8 @@
 #include "ecore_xcb_private.h"
 #include "Ecore_X_Atoms.h"
 
-static int _ecore_xcb_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
-static int _ecore_xcb_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler);
+static Eina_Bool _ecore_xcb_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
+static Eina_Bool _ecore_xcb_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler);
 static int _ecore_xcb_key_mask_get(xcb_keysym_t sym);
 static int _ecore_xcb_event_modifier(unsigned int state);
 
@@ -54,6 +54,7 @@ static xcb_generic_event_t *_ecore_xcb_event_buffered = NULL;
 
 static int _ecore_xcb_init_count = 0;
 static int _ecore_xcb_grab_count = 0;
+int _ecore_x11xcb_log_dom = -1;
 
 Ecore_X_Connection *_ecore_xcb_conn = NULL;
 Ecore_X_Screen     *_ecore_xcb_screen = NULL;
@@ -193,13 +194,20 @@ ecore_x_init(const char *name)
 
    xcb_intern_atom_cookie_t           atom_cookies[ECORE_X_ATOMS_COUNT];
 
-   if (--_ecore_xcb_init_count != 1)
+   if (++_ecore_xcb_init_count != 1)
      return _ecore_xcb_init_count;
-
+   _ecore_x11xcb_log_dom = eina_log_domain_register("EcoreXCB", ECORE_XLIB_XCB_DEFAULT_LOG_COLOR);
+   if(_ecore_x11xcb_log_dom < 0)
+     {
+       EINA_LOG_ERR("Impossible to create a log domain the Ecore XCB module.");
+       return --_ecore_xcb_init_count;
+     }
    _ecore_xcb_conn = xcb_connect(name, &screen);
-   if (!_ecore_xcb_conn)
+   if (xcb_connection_has_error(_ecore_xcb_conn)) {
+     eina_log_domain_unregister(_ecore_x11xcb_log_dom);
+     _ecore_x11xcb_log_dom = -1;
      return --_ecore_xcb_init_count;
-
+   }
    /* FIXME: no error code right now */
    /* _ecore_xcb_error_handler_init(); */
 
@@ -851,11 +859,14 @@ ecore_x_killall(Ecore_X_Window root)
         reply = xcb_query_tree_reply(_ecore_xcb_conn, cookie, NULL);
         if (reply)
           {
-            xcb_window_iterator_t iter;
+            xcb_window_t *wins = NULL;
+            int tree_c_len;
+            int i;
 
-            iter = xcb_query_tree_children_iterator(reply);
-            for (; iter.rem; xcb_window_next(&iter))
-               xcb_kill_client(_ecore_xcb_conn, *iter.data);
+            wins = xcb_query_tree_children(reply);
+            tree_c_len = xcb_query_tree_children_length(reply);
+            for (i = 0; i < tree_c_len; i++)
+              xcb_kill_client(_ecore_xcb_conn, wins[i]);
             free(reply);
           }
      }
@@ -901,7 +912,7 @@ handle_event(xcb_generic_event_t *ev)
      }
 }
 
-static int
+static Eina_Bool
 _ecore_xcb_fd_handler(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 {
    xcb_connection_t    *c;
@@ -909,19 +920,22 @@ _ecore_xcb_fd_handler(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 
    c = (xcb_connection_t *)data;
 
-/*    printf ("nbr events: %d\n", _ecore_xcb_event_handlers_num); */
+/*    INF ("nbr events: %d", _ecore_xcb_event_handlers_num); */
 
    /* We check if _ecore_xcb_event_buffered is NULL or not */
    if (_ecore_xcb_event_buffered)
-     handle_event(_ecore_xcb_event_buffered);
+     {
+        handle_event(_ecore_xcb_event_buffered);
+        _ecore_xcb_event_buffered = NULL;
+     }
 
    while ((ev = xcb_poll_for_event(c)))
      handle_event(ev);
 
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _ecore_xcb_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 {
    xcb_connection_t *c;
@@ -930,9 +944,9 @@ _ecore_xcb_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 
    _ecore_xcb_event_buffered = xcb_poll_for_event(c);
    if (!_ecore_xcb_event_buffered)
-     return 0;
+     return ECORE_CALLBACK_CANCEL;
 
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
 /* FIXME: possible roundtrip */
@@ -940,12 +954,13 @@ _ecore_xcb_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 static int
 _ecore_xcb_key_mask_get(xcb_keysym_t sym)
 {
-   xcb_keycode_iterator_t            iter;
    xcb_get_modifier_mapping_cookie_t cookie;
    xcb_get_modifier_mapping_reply_t *reply;
    xcb_key_symbols_t                *symbols;
    xcb_keysym_t                      sym2;
    int                               i, j;
+   xcb_keycode_t                    *keycodes = NULL;
+   int                               mod_keys_len;
    const int                         masks[8] =
      {
         XCB_MOD_MASK_SHIFT,
@@ -969,13 +984,13 @@ _ecore_xcb_key_mask_get(xcb_keysym_t sym)
         return 0;
      }
 
-   iter = xcb_get_modifier_mapping_keycodes_iterator(reply);
-
-   for (i = 0; iter.rem; xcb_keycode_next(&iter), i++)
+   keycodes = xcb_get_modifier_mapping_keycodes(reply);
+   mod_keys_len = xcb_get_modifier_mapping_keycodes_length(reply);
+   for (i = 0; i < mod_keys_len; i++)
      {
        for (j = 0; j < 8; j++)
          {
-            sym2 = xcb_key_symbols_get_keysym(symbols, *iter.data, j);
+            sym2 = xcb_key_symbols_get_keysym(symbols, keycodes[i], j);
             if (sym2 != 0) break;
          }
        if (sym2 == sym)
@@ -1492,13 +1507,13 @@ ecore_x_ungrab(void)
 
 int      _ecore_window_grabs_num = 0;
 Ecore_X_Window  *_ecore_window_grabs = NULL;
-int    (*_ecore_window_grab_replay_func) (void *data, int event_type, void *event);
+Eina_Bool (*_ecore_window_grab_replay_func) (void *data, int event_type, void *event);
 void    *_ecore_window_grab_replay_data;
 
 EAPI void
-ecore_x_passive_grab_replay_func_set(int (*func) (void *data,
-                                                  int   event_type,
-                                                  void *event),
+ecore_x_passive_grab_replay_func_set(Eina_Bool (*func) (void *data,
+							int   event_type,
+							void *event),
                                      void *data)
 {
    _ecore_window_grab_replay_func = func;
