@@ -8,16 +8,23 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
+
+//#define LOGRT 1
+
+#ifdef LOGRT
+#include <dlfcn.h>
+#endif
 
-#include "ecore_private.h"
 #include "Ecore.h"
+#include "ecore_private.h"
 #include "ecore_x_private.h"
 #include "Ecore_X.h"
 #include "Ecore_X_Atoms.h"
 #include "Ecore_Input.h"
 
-static int _ecore_x_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
-static int _ecore_x_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler);
+static Eina_Bool _ecore_x_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
+static Eina_Bool _ecore_x_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler);
 static int _ecore_x_key_mask_get(KeySym sym);
 static int _ecore_x_event_modifier(unsigned int state);
 
@@ -31,6 +38,7 @@ static const int AnyXEvent = 0; /* 0 can be used as there are no event types
 static int _ecore_x_event_shape_id = 0;
 static int _ecore_x_event_screensaver_id = 0;
 static int _ecore_x_event_sync_id = 0;
+int _ecore_xlib_log_dom = -1;
 
 #ifdef ECORE_XRANDR
 static int _ecore_x_event_randr_id = 0;
@@ -55,6 +63,7 @@ int      _ecore_x_event_last_root_x = 0;
 int      _ecore_x_event_last_root_y = 0;
 int      _ecore_x_xcursor = 0;
 XIC      _ecore_x_ic = NULL; /* Input context for composed characters */
+XIM      _ecore_x_im = NULL;
 
 Ecore_X_Window _ecore_x_private_win = 0;
 
@@ -118,6 +127,8 @@ EAPI int ECORE_X_EVENT_STARTUP_SEQUENCE_NEW = 0;
 EAPI int ECORE_X_EVENT_STARTUP_SEQUENCE_CHANGE = 0;
 EAPI int ECORE_X_EVENT_STARTUP_SEQUENCE_REMOVE = 0;
 
+EAPI int ECORE_X_EVENT_GENERIC = 0;
+
 int ECORE_X_MODIFIER_SHIFT = 0;
 int ECORE_X_MODIFIER_CTRL = 0;
 int ECORE_X_MODIFIER_ALT = 0;
@@ -127,6 +138,64 @@ EAPI int ECORE_X_LOCK_SCROLL = 0;
 EAPI int ECORE_X_LOCK_NUM = 0;
 EAPI int ECORE_X_LOCK_CAPS = 0;
 
+#ifdef LOGRT
+static double t0 = 0.0;
+static Status (*_logrt_real_reply) (Display *disp, void *rep, int extra, Bool discard) = NULL;
+static void
+_logrt_init(void)
+{
+   void *lib;
+   
+   lib = dlopen("libX11.so", RTLD_GLOBAL | RTLD_LAZY);
+   if (!lib) lib = dlopen("libX11.so.6", RTLD_GLOBAL | RTLD_LAZY);
+   if (!lib) lib = dlopen("libX11.so.6.3", RTLD_GLOBAL | RTLD_LAZY);
+   if (!lib) lib = dlopen("libX11.so.6.3.0", RTLD_GLOBAL | RTLD_LAZY);
+   _logrt_real_reply = dlsym(lib, "_XReply");
+   t0 = ecore_time_get();
+}
+Status
+_XReply(Display *disp, void *rep, int extra, Bool discard)
+{
+   void *bt[128];
+   int i, n;
+   char **sym;
+   
+   n = backtrace(bt, 128);
+   if (n > 0)
+     {
+        sym = backtrace_symbols(bt, n);
+        printf("ROUNDTRIP: %4.4f :", ecore_time_get() - t0);
+        if (sym)
+          {
+             for (i = n - 1; i > 0; i--)
+               {
+                  char *fname = strchr(sym[i], '(');
+                  if (fname)
+                    {
+                       char *tsym = alloca(strlen(fname) + 1);
+                       char *end;
+                       strcpy(tsym, fname + 1);
+                       end = strchr(tsym, '+');
+                       if (end)
+                         {
+                            *end = 0;
+                            printf("%s", tsym);
+                         }
+                       else
+                         printf("???");
+                    }
+                  else
+                    printf("???");
+                  if (i > 1) printf(" > ");
+               }
+             printf("\n");
+          }
+     }
+   // fixme: logme
+   return _logrt_real_reply(disp, rep, extra, discard);
+}
+#endif
+
 /**
  * @defgroup Ecore_X_Init_Group X Library Init and Shutdown Functions
  *
@@ -169,9 +238,26 @@ ecore_x_init(const char *name)
    if (++_ecore_x_init_count != 1) 
      return _ecore_x_init_count;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+#ifdef LOGRT
+   _logrt_init();
+#endif   
+   
+   _ecore_xlib_log_dom = eina_log_domain_register("EcoreX11", ECORE_XLIB_DEFAULT_LOG_COLOR);
+   if(_ecore_xlib_log_dom < 0)
+     {
+       EINA_LOG_ERR("Impossible to create a log domain for the Ecore Xlib module.");
+       return --_ecore_x_init_count;
+     }
    if (!ecore_event_init())
-     return --_ecore_x_init_count;
-
+     {
+       eina_log_domain_unregister(_ecore_xlib_log_dom);
+	_ecore_xlib_log_dom = -1;
+       return --_ecore_x_init_count;
+     }
+#ifdef EVAS_FRAME_QUEUING
+   XInitThreads();
+#endif
    _ecore_x_disp = XOpenDisplay((char *)name);
    if (!_ecore_x_disp)
      goto shutdown_ecore_event;
@@ -263,6 +349,10 @@ ecore_x_init(const char *name)
    _ecore_x_event_handlers[ColormapNotify]   = _ecore_x_event_handle_colormap_notify;
    _ecore_x_event_handlers[ClientMessage]    = _ecore_x_event_handle_client_message;
    _ecore_x_event_handlers[MappingNotify]    = _ecore_x_event_handle_mapping_notify;
+#ifdef GenericEvent
+  _ecore_x_event_handlers[GenericEvent]    = _ecore_x_event_handle_generic_event;	
+#endif
+   
    if (_ecore_x_event_shape_id)
      _ecore_x_event_handlers[_ecore_x_event_shape_id] = _ecore_x_event_handle_shape_change;
    if (_ecore_x_event_screensaver_id)
@@ -362,6 +452,8 @@ ecore_x_init(const char *name)
 	ECORE_X_EVENT_STARTUP_SEQUENCE_NEW     = ecore_event_type_new();
 	ECORE_X_EVENT_STARTUP_SEQUENCE_CHANGE  = ecore_event_type_new();
 	ECORE_X_EVENT_STARTUP_SEQUENCE_REMOVE  = ecore_event_type_new();
+
+	ECORE_X_EVENT_GENERIC                  = ecore_event_type_new();
      }
    
    /* everything has these... unless its like a pda... :) */
@@ -424,6 +516,7 @@ ecore_x_init(const char *name)
    _ecore_x_composite_init();
    _ecore_x_dpms_init();
    _ecore_x_randr_init();
+   _ecore_x_input_init();
    
    _ecore_x_private_win = ecore_x_window_override_new(0, -77, -777, 123, 456);
 
@@ -446,8 +539,11 @@ ecore_x_init(const char *name)
 	if (ret || !supported_styles)
 	  goto _im_create_error;
 	for (i = 0; i < supported_styles->count_styles; i++)
-	  if (supported_styles->supported_styles[i] == (XIMPreeditNothing | XIMStatusNothing))
-	    chosen_style = supported_styles->supported_styles[i];
+          {
+             if (supported_styles->supported_styles[i] == 
+                 (XIMPreeditNothing | XIMStatusNothing))
+               chosen_style = supported_styles->supported_styles[i];
+          }
 	XFree(supported_styles);
 	if (!chosen_style)
 	  goto _im_create_error;
@@ -455,6 +551,7 @@ ecore_x_init(const char *name)
 	if (ic)
 	  {
 	     _ecore_x_ic = ic;
+             _ecore_x_im = im;
 	     goto _im_create_end;
 	  }
 _im_create_error:
@@ -464,16 +561,15 @@ _im_create_end:
 #endif
    return _ecore_x_init_count;
 
- free_event_handlers:
+free_event_handlers:
    free(_ecore_x_event_handlers);
    _ecore_x_event_handlers = NULL;
- close_display:
+close_display:
    XCloseDisplay(_ecore_x_disp);
    _ecore_x_fd_handler_handle = NULL;
    _ecore_x_disp = NULL;
- shutdown_ecore_event:
+shutdown_ecore_event:
    ecore_event_shutdown();
-
    return --_ecore_x_init_count;
 }
 
@@ -484,16 +580,19 @@ _ecore_x_shutdown(int close_display)
      return _ecore_x_init_count;
 
    if (!_ecore_x_disp) return _ecore_x_init_count;
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
 
 #ifdef ENABLE_XIM
    if (_ecore_x_ic)
      {
-	XIM xim;
-	xim = XIMOfIC(_ecore_x_ic);
 	XDestroyIC(_ecore_x_ic);
-	XCloseIM(xim);
 	_ecore_x_ic = NULL;
      }
+   if (_ecore_x_im)
+     {
+	XCloseIM(_ecore_x_im);
+        _ecore_x_im = NULL;
+     }
 #endif
    if (close_display)
       XCloseDisplay(_ecore_x_disp);
@@ -504,11 +603,13 @@ _ecore_x_shutdown(int close_display)
    _ecore_x_fd_handler_handle = NULL;
    _ecore_x_disp = NULL;
    _ecore_x_event_handlers = NULL;
+   _ecore_x_input_shutdown();
    _ecore_x_selection_shutdown();
    _ecore_x_dnd_shutdown();
    ecore_x_netwm_shutdown();
    ecore_event_shutdown();
-
+   eina_log_domain_unregister(_ecore_xlib_log_dom);
+   _ecore_xlib_log_dom = -1;
    return _ecore_x_init_count;
 }
 
@@ -566,6 +667,7 @@ ecore_x_display_get(void)
 EAPI int
 ecore_x_fd_get(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ConnectionNumber(_ecore_x_disp);
 }
 
@@ -577,6 +679,7 @@ ecore_x_fd_get(void)
 EAPI Ecore_X_Screen*
 ecore_x_default_screen_get(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return (Ecore_X_Screen*) DefaultScreenOfDisplay(_ecore_x_disp);
 }
 
@@ -625,6 +728,7 @@ ecore_x_double_click_time_get(void)
 EAPI void
 ecore_x_flush(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFlush(_ecore_x_disp);
 }
 
@@ -636,6 +740,7 @@ ecore_x_flush(void)
 EAPI void
 ecore_x_sync(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSync(_ecore_x_disp, False);
 }
 
@@ -657,6 +762,7 @@ ecore_x_killall(Ecore_X_Window root)
    Window *children_r = NULL;
    unsigned int num_children = 0;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGrabServer(_ecore_x_disp);
    /* Tranverse window tree starting from root, and drag each
     * before the firing squad */
@@ -684,6 +790,7 @@ ecore_x_killall(Ecore_X_Window root)
 EAPI void
 ecore_x_kill(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XKillClient(_ecore_x_disp, win);
 }
 
@@ -710,12 +817,13 @@ ecore_x_dpi_get(void)
 {
    Screen *s;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    s = DefaultScreenOfDisplay(_ecore_x_disp);
    if (s->mwidth <= 0) return 75;
    return (((s->width * 254) / s->mwidth) + 5) / 10;
 }
 
-static int
+static Eina_Bool
 _ecore_x_fd_handler(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 {
    Display *d;
@@ -741,17 +849,17 @@ _ecore_x_fd_handler(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 	       _ecore_x_event_handlers[ev.type] (&ev);
 	  }
      }
-   return 1;
+   return ECORE_CALLBACK_RENEW;
 }
 
-static int
+static Eina_Bool
 _ecore_x_fd_handler_buf(void *data, Ecore_Fd_Handler *fd_handler __UNUSED__)
 {
    Display *d;
 
    d = data;
-   if (XPending(d)) return 1;
-   return 0;
+   if (XPending(d)) return ECORE_CALLBACK_RENEW;
+   return ECORE_CALLBACK_CANCEL;
 }
 
 static int
@@ -842,6 +950,7 @@ ecore_x_window_root_list(int *num_ret)
    if (!num_ret) return NULL;
    *num_ret = 0;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
 #ifdef ECORE_XPRINT
    num = ScreenCount(_ecore_x_disp);
    if (XpQueryExtension(_ecore_x_disp, &xp_base, &xp_err_base))
@@ -923,19 +1032,23 @@ ecore_x_window_root_list(int *num_ret)
 EAPI Ecore_X_Window
 ecore_x_window_root_first_get(void)
 {
+   return RootWindow(_ecore_x_disp, 0);
+/*   
    int num;
    Ecore_X_Window root, *roots = NULL;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    roots = ecore_x_window_root_list(&num);
-   if(!(roots)) return 0;
+   if (!(roots)) return 0;
    
    if (num > 0)
-      root = roots[0];
+     root = roots[0];
    else
-      root = 0;
-
+     root = 0;
+   
    free(roots);
    return root;
+ */
 }
 
 
@@ -955,6 +1068,7 @@ ecore_x_window_manage(Ecore_X_Window win)
 {
    XWindowAttributes   att;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XGetWindowAttributes(_ecore_x_disp, win, &att) != True) return 0;
    ecore_x_sync();
    _ecore_x_window_manage_failed = 0;
@@ -983,6 +1097,7 @@ ecore_x_window_manage(Ecore_X_Window win)
 EAPI void
 ecore_x_window_container_manage(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSelectInput(_ecore_x_disp, win, 
 		SubstructureRedirectMask | 
 		SubstructureNotifyMask);
@@ -991,13 +1106,15 @@ ecore_x_window_container_manage(Ecore_X_Window win)
 EAPI void
 ecore_x_window_client_manage(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSelectInput(_ecore_x_disp, win, 
 		PropertyChangeMask | 
 //		ResizeRedirectMask |
 		FocusChangeMask |
 		ColormapChangeMask |
 		VisibilityChangeMask |
-		StructureNotifyMask
+		StructureNotifyMask |
+                SubstructureNotifyMask
 		);
    XShapeSelectInput(_ecore_x_disp, win, ShapeNotifyMask);
 }
@@ -1005,6 +1122,7 @@ ecore_x_window_client_manage(Ecore_X_Window win)
 EAPI void
 ecore_x_window_sniff(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSelectInput(_ecore_x_disp, win,
 		PropertyChangeMask |
 		SubstructureNotifyMask);
@@ -1013,12 +1131,14 @@ ecore_x_window_sniff(Ecore_X_Window win)
 EAPI void
 ecore_x_window_client_sniff(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSelectInput(_ecore_x_disp, win,
 		PropertyChangeMask |
 		FocusChangeMask |
 		ColormapChangeMask |
 		VisibilityChangeMask |
-		StructureNotifyMask);
+		StructureNotifyMask |
+                SubstructureNotifyMask);
    XShapeSelectInput(_ecore_x_disp, win, ShapeNotifyMask);
 }
 
@@ -1032,6 +1152,7 @@ ecore_x_window_attributes_get(Ecore_X_Window win, Ecore_X_Window_Attributes *att
 {
    XWindowAttributes att;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetWindowAttributes(_ecore_x_disp, win, &att)) return 0;
    memset(att_ret, 0, sizeof(Ecore_X_Window_Attributes));
    att_ret->root = att.root;
@@ -1059,12 +1180,14 @@ ecore_x_window_attributes_get(Ecore_X_Window win, Ecore_X_Window_Attributes *att
 EAPI void
 ecore_x_window_save_set_add(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XAddToSaveSet(_ecore_x_disp, win);
 }
 
 EAPI void
 ecore_x_window_save_set_del(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XRemoveFromSaveSet(_ecore_x_disp, win);
 }
 
@@ -1075,6 +1198,7 @@ ecore_x_window_children_get(Ecore_X_Window win, int *num)
    Window              root_ret = 0, parent_ret = 0, *children_ret = NULL;
    unsigned int        children_ret_num = 0;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XQueryTree(_ecore_x_disp, win, &root_ret, &parent_ret, &children_ret,
 		   &children_ret_num))
      {
@@ -1099,6 +1223,7 @@ ecore_x_window_children_get(Ecore_X_Window win, int *num)
 EAPI int
 ecore_x_pointer_control_set(int accel_num, int accel_denom, int threshold)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XChangePointerControl(_ecore_x_disp, 1, 1, 
 				accel_num, accel_denom, threshold);
 }
@@ -1106,6 +1231,7 @@ ecore_x_pointer_control_set(int accel_num, int accel_denom, int threshold)
 EAPI int
 ecore_x_pointer_control_get(int *accel_num, int *accel_denom, int *threshold)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XGetPointerControl(_ecore_x_disp, 
 				accel_num, accel_denom, threshold);
 }
@@ -1113,18 +1239,21 @@ ecore_x_pointer_control_get(int *accel_num, int *accel_denom, int *threshold)
 EAPI int
 ecore_x_pointer_mapping_set(unsigned char *map, int nmap)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XSetPointerMapping(_ecore_x_disp, map, nmap);
 }
 
 EAPI int
 ecore_x_pointer_mapping_get(unsigned char *map, int nmap)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XGetPointerMapping(_ecore_x_disp, map, nmap);
 }
 
 EAPI int
 ecore_x_pointer_grab(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XGrabPointer(_ecore_x_disp, win, False,
 		    ButtonPressMask | ButtonReleaseMask | 
 		    EnterWindowMask | LeaveWindowMask | PointerMotionMask,
@@ -1136,6 +1265,7 @@ ecore_x_pointer_grab(Ecore_X_Window win)
 EAPI int
 ecore_x_pointer_confine_grab(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XGrabPointer(_ecore_x_disp, win, False,
 		    ButtonPressMask | ButtonReleaseMask | 
 		    EnterWindowMask | LeaveWindowMask | PointerMotionMask,
@@ -1147,18 +1277,21 @@ ecore_x_pointer_confine_grab(Ecore_X_Window win)
 EAPI void
 ecore_x_pointer_ungrab(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XUngrabPointer(_ecore_x_disp, CurrentTime);
 }
 
 EAPI int
 ecore_x_pointer_warp(Ecore_X_Window win, int x, int y)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return XWarpPointer(_ecore_x_disp, None, win, 0, 0, 0, 0, x, y);
 }
 
 EAPI int
 ecore_x_keyboard_grab(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XGrabKeyboard(_ecore_x_disp, win, False,
 		     GrabModeAsync, GrabModeAsync,
 		     CurrentTime) == GrabSuccess) return 1;
@@ -1168,40 +1301,36 @@ ecore_x_keyboard_grab(Ecore_X_Window win)
 EAPI void
 ecore_x_keyboard_ungrab(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XUngrabKeyboard(_ecore_x_disp, CurrentTime);   
 }
 
 EAPI void
 ecore_x_grab(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_x_grab_count++;
-   
-   if (_ecore_x_grab_count == 1)
-      XGrabServer(_ecore_x_disp);
+   if (_ecore_x_grab_count == 1) XGrabServer(_ecore_x_disp);
 }
 
 EAPI void
 ecore_x_ungrab(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_x_grab_count--;
-   if (_ecore_x_grab_count < 0)
-      _ecore_x_grab_count = 0;
-
-   if (_ecore_x_grab_count == 0)
-   {
-      XUngrabServer(_ecore_x_disp);
-      XSync(_ecore_x_disp, False);
-   }
+   if (_ecore_x_grab_count < 0) _ecore_x_grab_count = 0;
+   if (_ecore_x_grab_count == 0) XUngrabServer(_ecore_x_disp);
 }
 
 int      _ecore_window_grabs_num = 0;
 Window  *_ecore_window_grabs = NULL;
-int    (*_ecore_window_grab_replay_func) (void *data, int event_type, void *event);
+Eina_Bool (*_ecore_window_grab_replay_func) (void *data, int event_type, void *event);
 void    *_ecore_window_grab_replay_data;
 
 EAPI void
-ecore_x_passive_grab_replay_func_set(int (*func) (void *data, int event_type, void *event), void *data)
+ecore_x_passive_grab_replay_func_set(Eina_Bool (*func) (void *data, int event_type, void *event), void *data)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    _ecore_window_grab_replay_func = func;
    _ecore_window_grab_replay_data = data;
 }
@@ -1216,6 +1345,7 @@ ecore_x_window_button_grab(Ecore_X_Window win, int button,
    unsigned int        locks[8];
    int                 i, ev;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    b = button;
    if (b == 0) b = AnyButton;
    m = _ecore_x_event_modifier(mod);
@@ -1287,6 +1417,7 @@ ecore_x_window_button_ungrab(Ecore_X_Window win, int button,
    unsigned int        locks[8];
    int                 i;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    b = button;
    if (b == 0) b = AnyButton;
    m = _ecore_x_event_modifier(mod);
@@ -1317,6 +1448,7 @@ ecore_x_window_key_grab(Ecore_X_Window win, const char *key,
    unsigned int        locks[8];
    int                 i;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strncmp(key, "Keycode-", 8))
      keycode = atoi(key + 8);
    else
@@ -1378,6 +1510,7 @@ ecore_x_window_key_ungrab(Ecore_X_Window win, const char *key,
    unsigned int        locks[8];
    int                 i;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!strncmp(key, "Keycode-", 8))
      keycode = atoi(key + 8);
    else
@@ -1421,19 +1554,20 @@ ecore_x_client_message32_send(Ecore_X_Window win, Ecore_X_Atom type,
 			      Ecore_X_Event_Mask mask,
 			      long d0, long d1, long d2, long d3, long d4)
 {
-    XEvent xev;
+   XEvent xev;
 
-    xev.xclient.window = win;
-    xev.xclient.type = ClientMessage;
-    xev.xclient.message_type = type;
-    xev.xclient.format = 32;
-    xev.xclient.data.l[0] = d0;
-    xev.xclient.data.l[1] = d1;
-    xev.xclient.data.l[2] = d2;
-    xev.xclient.data.l[3] = d3;
-    xev.xclient.data.l[4] = d4;
-
-    return XSendEvent(_ecore_x_disp, win, False, mask, &xev);
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   xev.xclient.window = win;
+   xev.xclient.type = ClientMessage;
+   xev.xclient.message_type = type;
+   xev.xclient.format = 32;
+   xev.xclient.data.l[0] = d0;
+   xev.xclient.data.l[1] = d1;
+   xev.xclient.data.l[2] = d2;
+   xev.xclient.data.l[3] = d3;
+   xev.xclient.data.l[4] = d4;
+   
+   return XSendEvent(_ecore_x_disp, win, False, mask, &xev);
 }
 
 /**
@@ -1450,18 +1584,18 @@ EAPI int
 ecore_x_client_message8_send(Ecore_X_Window win, Ecore_X_Atom type,
 			     const void *data, int len)
 {
-    XEvent xev;
-
-    xev.xclient.window = win;
-    xev.xclient.type = ClientMessage;
-    xev.xclient.message_type = type;
-    xev.xclient.format = 8;
-    if (len > 20)
-        len = 20;
-    memcpy(xev.xclient.data.b, data, len);
-    memset(xev.xclient.data.b + len, 0, 20 - len);
-
-    return XSendEvent(_ecore_x_disp, win, False, NoEventMask, &xev);
+   XEvent xev;
+   
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   xev.xclient.window = win;
+   xev.xclient.type = ClientMessage;
+   xev.xclient.message_type = type;
+   xev.xclient.format = 8;
+   if (len > 20) len = 20;
+   memcpy(xev.xclient.data.b, data, len);
+   memset(xev.xclient.data.b + len, 0, 20 - len);
+   
+   return XSendEvent(_ecore_x_disp, win, False, NoEventMask, &xev);
 }
 
 EAPI int
@@ -1472,6 +1606,7 @@ ecore_x_mouse_move_send(Ecore_X_Window win, int x, int y)
    Window tw;
    int rx, ry;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetWindowAttributes(_ecore_x_disp, win, &att);
    XTranslateCoordinates(_ecore_x_disp, win, att.root, x, y, &rx, &ry, &tw);
    xev.xmotion.type = MotionNotify;
@@ -1497,6 +1632,7 @@ ecore_x_mouse_down_send(Ecore_X_Window win, int x, int y, int b)
    Window tw;
    int rx, ry;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetWindowAttributes(_ecore_x_disp, win, &att);
    XTranslateCoordinates(_ecore_x_disp, win, att.root, x, y, &rx, &ry, &tw);
    xev.xbutton.type = ButtonPress;
@@ -1522,6 +1658,7 @@ ecore_x_mouse_up_send(Ecore_X_Window win, int x, int y, int b)
    Window tw;
    int rx, ry;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XGetWindowAttributes(_ecore_x_disp, win, &att);
    XTranslateCoordinates(_ecore_x_disp, win, att.root, x, y, &rx, &ry, &tw);
    xev.xbutton.type = ButtonRelease;
@@ -1542,12 +1679,14 @@ ecore_x_mouse_up_send(Ecore_X_Window win, int x, int y, int b)
 EAPI void
 ecore_x_focus_reset(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSetInputFocus(_ecore_x_disp, PointerRoot, RevertToPointerRoot, CurrentTime);
 }
 
 EAPI void
 ecore_x_events_allow_all(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XAllowEvents(_ecore_x_disp, AsyncBoth, CurrentTime);
 }
 
@@ -1565,6 +1704,7 @@ ecore_x_pointer_xy_get(Ecore_X_Window win, int *x, int *y)
    int rx, ry, wx, wy, ret;
    unsigned int mask;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ret = XQueryPointer(_ecore_x_disp, win, &rwin, &cwin, &rx, &ry, &wx, &wy, &mask);
    if (!ret) wx = wy = -1;
    if (x) *x = wx;
