@@ -25,6 +25,7 @@
 EAPI void
 ecore_x_icccm_init(void)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
 }
 
 EAPI void
@@ -32,6 +33,7 @@ ecore_x_icccm_state_set(Ecore_X_Window win, Ecore_X_Window_State_Hint state)
 {
    unsigned long       c[2];
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (state == ECORE_X_WINDOW_STATE_HINT_WITHDRAWN)
       c[0] = WithdrawnState;
    else if (state == ECORE_X_WINDOW_STATE_HINT_NORMAL)
@@ -53,6 +55,7 @@ ecore_x_icccm_state_get(Ecore_X_Window win)
    int            format_ret;
    Ecore_X_Window_State_Hint hint;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    hint = ECORE_X_WINDOW_STATE_HINT_NONE;
    XGetWindowProperty(_ecore_x_disp, win, ECORE_X_ATOM_WM_STATE,
 		      0, 0x7fffffff, False, ECORE_X_ATOM_WM_STATE,
@@ -77,6 +80,7 @@ ecore_x_icccm_state_get(Ecore_X_Window win)
 EAPI void
 ecore_x_icccm_delete_window_send(Ecore_X_Window win, Ecore_X_Time t)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_client_message32_send(win, ECORE_X_ATOM_WM_PROTOCOLS,
 				 ECORE_X_EVENT_MASK_NONE,
 				 ECORE_X_ATOM_WM_DELETE_WINDOW,
@@ -86,6 +90,7 @@ ecore_x_icccm_delete_window_send(Ecore_X_Window win, Ecore_X_Time t)
 EAPI void
 ecore_x_icccm_take_focus_send(Ecore_X_Window win, Ecore_X_Time t)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_client_message32_send(win, ECORE_X_ATOM_WM_PROTOCOLS,
 				 ECORE_X_EVENT_MASK_NONE,
 				 ECORE_X_ATOM_WM_TAKE_FOCUS,
@@ -95,6 +100,7 @@ ecore_x_icccm_take_focus_send(Ecore_X_Window win, Ecore_X_Time t)
 EAPI void
 ecore_x_icccm_save_yourself_send(Ecore_X_Window win, Ecore_X_Time t)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_client_message32_send(win, ECORE_X_ATOM_WM_PROTOCOLS,
 				 ECORE_X_EVENT_MASK_NONE,
 				 ECORE_X_ATOM_WM_SAVE_YOURSELF,
@@ -104,8 +110,9 @@ ecore_x_icccm_save_yourself_send(Ecore_X_Window win, Ecore_X_Time t)
 EAPI void
 ecore_x_icccm_move_resize_send(Ecore_X_Window win, int x, int y, int w, int h)
 {
-   XEvent              ev;
+   XEvent ev;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ev.type = ConfigureNotify;
    ev.xconfigure.display = _ecore_x_disp;
    ev.xconfigure.event = win;
@@ -135,6 +142,7 @@ ecore_x_icccm_hints_set(Ecore_X_Window win,
    if (!hints)
       return;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    hints->flags = InputHint | StateHint;
    hints->input = accepts_focus;
    if (initial_state == ECORE_X_WINDOW_STATE_HINT_WITHDRAWN)
@@ -180,6 +188,7 @@ ecore_x_icccm_hints_get(Ecore_X_Window win,
 {
    XWMHints *hints;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (accepts_focus)
       *accepts_focus = 1;
    if (initial_state)
@@ -252,6 +261,7 @@ ecore_x_icccm_size_pos_hints_set(Ecore_X_Window win,
    XSizeHints  hint;
    long        mask;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetWMNormalHints(_ecore_x_disp, win, &hint, &mask))
      {
 	memset(&hint, 0, sizeof(XSizeHints));
@@ -321,6 +331,7 @@ ecore_x_icccm_size_pos_hints_get(Ecore_X_Window win,
    int                 stepx = -1, stepy = -1;
    double              mina = 0.0, maxa = 0.0;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetWMNormalHints(_ecore_x_disp, win, &hint, &mask))
      return 0;
 
@@ -415,6 +426,7 @@ ecore_x_icccm_title_set(Ecore_X_Window win, const char *t)
 
    if (!t) return;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    xprop.value = NULL;
 #ifdef X_HAVE_UTF8_STRING
    list[0] = strdup(t);
@@ -443,20 +455,21 @@ ecore_x_icccm_title_set(Ecore_X_Window win, const char *t)
    free(list[0]);
 }
 
-EAPI char               *
+EAPI char *
 ecore_x_icccm_title_get(Ecore_X_Window win)
 {
    XTextProperty       xprop;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    xprop.value = NULL;
    if (XGetWMName(_ecore_x_disp, win, &xprop) >= Success)
      {
 	if (xprop.value)
 	  {
-	     char              **list = NULL;
-	     char               *t = NULL;
-	     int                 num = 0;
-	     int                 ret;
+	     char **list = NULL;
+	     char *t = NULL;
+	     int num = 0;
+	     int ret;
 
 	     if (xprop.encoding == ECORE_X_ATOM_UTF8_STRING)
 	       {
@@ -494,6 +507,22 @@ ecore_x_icccm_title_get(Ecore_X_Window win)
    return NULL;
 }
 
+/**
+ * Set protocol atoms explicitly
+ * @param win The Window
+ * @param protos An array of protocol atoms
+ * @param num the number of members of the array
+ */
+EAPI void
+ecore_x_icccm_protocol_atoms_set(Ecore_X_Window win, Ecore_X_Atom *protos, int num)
+{
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   if (num > 0)
+     XSetWMProtocols(_ecore_x_disp, win, (Atom *)(protos), num);
+   else
+     XDeleteProperty(_ecore_x_disp, win, ECORE_X_ATOM_WM_PROTOCOLS);
+}
+
 /**
  * Set or unset a wm protocol property.
  * @param win The Window
@@ -514,6 +543,7 @@ ecore_x_icccm_protocol_set(Ecore_X_Window win,
    if (protocol >= ECORE_X_WM_PROTOCOL_NUM)
       return;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    proto = _ecore_x_atoms_wm_protocols[protocol];
 
    if (!XGetWMProtocols(_ecore_x_disp, win, &protos, &protos_count))
@@ -591,6 +621,7 @@ ecore_x_icccm_protocol_isset(Ecore_X_Window win, Ecore_X_WM_Protocol protocol)
    if (protocol >= ECORE_X_WM_PROTOCOL_NUM)
       return 0;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    proto = _ecore_x_atoms_wm_protocols[protocol];
 
    if (!XGetWMProtocols(_ecore_x_disp, win, &protos, &protos_count))
@@ -624,6 +655,7 @@ ecore_x_icccm_name_class_set(Ecore_X_Window win, const char *n, const char *c)
    xch = XAllocClassHint();
    if (!xch)
       return;
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    xch->res_name = (char *)n;
    xch->res_class = (char *)c;
    XSetClassHint(_ecore_x_disp, win, xch);
@@ -643,6 +675,7 @@ ecore_x_icccm_name_class_get(Ecore_X_Window win, char **n, char **c)
 {
    XClassHint          xch;
    
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (n) *n = NULL;
    if (c) *c = NULL;
    xch.res_name = NULL;
@@ -674,6 +707,7 @@ ecore_x_icccm_client_machine_get(Ecore_X_Window win)
 {
    char               *name;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    name = ecore_x_window_prop_string_get(win, ECORE_X_ATOM_WM_CLIENT_MACHINE);
    return name;
 }
@@ -688,6 +722,7 @@ ecore_x_icccm_client_machine_get(Ecore_X_Window win)
 EAPI void
 ecore_x_icccm_command_set(Ecore_X_Window win, int argc, char **argv)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSetCommand(_ecore_x_disp, win, argv, argc);
 }
 
@@ -709,6 +744,7 @@ ecore_x_icccm_command_get(Ecore_X_Window win, int *argc, char ***argv)
    if (argc) *argc = 0;
    if (argv) *argv = NULL;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!XGetCommand(_ecore_x_disp, win, &v, &c))
      return;
    if (c < 1)
@@ -753,6 +789,7 @@ ecore_x_icccm_icon_name_set(Ecore_X_Window win, const char *t)
    XTextProperty       xprop;
    int                 ret;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    xprop.value = NULL;
 #ifdef X_HAVE_UTF8_STRING
    list[0] = strdup(t);
@@ -791,6 +828,7 @@ ecore_x_icccm_icon_name_get(Ecore_X_Window win)
 {
    XTextProperty       xprop;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    xprop.value = NULL;
    if (XGetWMIconName(_ecore_x_disp, win, &xprop) >= Success)
      {
@@ -854,6 +892,7 @@ ecore_x_icccm_colormap_window_set(Ecore_X_Window win, Ecore_X_Window subwin)
    Window             *oldset = NULL;
    Window             *newset = NULL;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!ecore_x_window_prop_property_get(win,
 					 ECORE_X_ATOM_WM_COLORMAP_WINDOWS,
 					 XA_WINDOW, 32, &old_data, &num))
@@ -909,6 +948,7 @@ ecore_x_icccm_colormap_window_unset(Ecore_X_Window win, Ecore_X_Window subwin)
    Window             *oldset = NULL;
    Window             *newset = NULL;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!ecore_x_window_prop_property_get(win,
 					 ECORE_X_ATOM_WM_COLORMAP_WINDOWS,
 					 XA_WINDOW, 32, &old_data, &num))
@@ -956,6 +996,7 @@ ecore_x_icccm_colormap_window_unset(Ecore_X_Window win, Ecore_X_Window subwin)
 EAPI void
 ecore_x_icccm_transient_for_set(Ecore_X_Window win, Ecore_X_Window forwin)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XSetTransientForHint(_ecore_x_disp, win, forwin);
 }
 
@@ -966,6 +1007,7 @@ ecore_x_icccm_transient_for_set(Ecore_X_Window win, Ecore_X_Window forwin)
 EAPI void
 ecore_x_icccm_transient_for_unset(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XDeleteProperty(_ecore_x_disp, win, ECORE_X_ATOM_WM_TRANSIENT_FOR);
 }
 
@@ -979,10 +1021,11 @@ ecore_x_icccm_transient_for_get(Ecore_X_Window win)
 {
    Window              forwin;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (XGetTransientForHint(_ecore_x_disp, win, &forwin))
-      return (Ecore_X_Window) forwin;
+     return (Ecore_X_Window) forwin;
    else
-      return 0;
+     return 0;
 
 }
 
@@ -994,6 +1037,7 @@ ecore_x_icccm_transient_for_get(Ecore_X_Window win)
 EAPI void
 ecore_x_icccm_window_role_set(Ecore_X_Window win, const char *role)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_window_prop_string_set(win, ECORE_X_ATOM_WM_WINDOW_ROLE,
 				  (char *)role);
 }
@@ -1006,6 +1050,7 @@ ecore_x_icccm_window_role_set(Ecore_X_Window win, const char *role)
 EAPI char               *
 ecore_x_icccm_window_role_get(Ecore_X_Window win)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_window_prop_string_get(win, ECORE_X_ATOM_WM_WINDOW_ROLE);
 }
 
@@ -1020,6 +1065,7 @@ ecore_x_icccm_window_role_get(Ecore_X_Window win)
 EAPI void
 ecore_x_icccm_client_leader_set(Ecore_X_Window win, Ecore_X_Window l)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    ecore_x_window_prop_window_set(win, ECORE_X_ATOM_WM_CLIENT_LEADER,
 				  &l, 1);
 }
@@ -1033,9 +1079,10 @@ ecore_x_icccm_client_leader_get(Ecore_X_Window win)
 {
    Ecore_X_Window      l;
 
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (ecore_x_window_prop_window_get(win, ECORE_X_ATOM_WM_CLIENT_LEADER,
 				      &l, 1) > 0)
-      return l;
+     return l;
    return 0;
 }
 
@@ -1045,6 +1092,7 @@ ecore_x_icccm_iconic_request_send(Ecore_X_Window win, Ecore_X_Window root)
    XEvent xev;
 
    if (!win) return;
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (!root) root = DefaultRootWindow(_ecore_x_disp);
 
    xev.xclient.type = ClientMessage;
