@@ -31,6 +31,7 @@
 EAPI Ecore_X_Pixmap
 ecore_x_pixmap_new(Ecore_X_Window win, int w, int h, int dep)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    if (win == 0) win = DefaultRootWindow(_ecore_x_disp);
    if (dep == 0) dep = DefaultDepth(_ecore_x_disp, DefaultScreen(_ecore_x_disp));
    return XCreatePixmap(_ecore_x_disp, win, w, h, dep);
@@ -48,6 +49,7 @@ ecore_x_pixmap_new(Ecore_X_Window win, int w, int h, int dep)
 EAPI void
 ecore_x_pixmap_free(Ecore_X_Pixmap pmap)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XFreePixmap(_ecore_x_disp, pmap);
 }
 
@@ -70,6 +72,7 @@ ecore_x_pixmap_paste(Ecore_X_Pixmap pmap, Ecore_X_Drawable dest,
 		     Ecore_X_GC gc, int sx, int sy, 
 		     int w, int h, int dx, int dy)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    XCopyArea(_ecore_x_disp, pmap, dest, gc, sx, sy, w, h, dx, dy);
 }
 
@@ -85,8 +88,8 @@ ecore_x_pixmap_paste(Ecore_X_Pixmap pmap, Ecore_X_Drawable dest,
 EAPI void
 ecore_x_pixmap_geometry_get(Ecore_X_Pixmap pmap, int *x, int *y, int *w, int *h)
 {
-   if (pmap)
-      ecore_x_drawable_geometry_get(pmap, x, y, w, h);
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
+   if (pmap) ecore_x_drawable_geometry_get(pmap, x, y, w, h);
 }
 
 /**
@@ -98,6 +101,7 @@ ecore_x_pixmap_geometry_get(Ecore_X_Pixmap pmap, int *x, int *y, int *w, int *h)
 EAPI int
 ecore_x_pixmap_depth_get(Ecore_X_Pixmap pmap)
 {
+   LOGFN(__FILE__, __LINE__, __FUNCTION__);
    return ecore_x_drawable_depth_get(pmap);
 }
 
